#include "Dashboard.h"
#include "Gauge.h"
#include "AnalogGauge.h"
#include "TextualGauge.h"
#include "Scene_Template.h"
#include "Scene.h"
#include "Default.h"
#include "xmlParser.h"
//#include "ECUScan.h"
//#include "Diagnostic.h"
#include "DataLogger.h"

#include <cairomm/context.h>
#include <glibmm/thread.h>
#include <math.h>
#include <string>
#include <iostream> 
#include <queue>
#include <time.h>
#include <string>
#include <fstream>
using namespace std;

Dashboard::Dashboard(void) {
    // Intialize and create the threads in here
    // The thread will point to the process command function in each of
    // The hardware modules
    data_logger = new DataLogger("/dev/ttyACM0");
    
    this->initialize = false;
    this->nScenes = 0;
    this->tempInfo = NULL;
    this->currentScene = NULL;
    Glib::signal_timeout().connect(sigc::mem_fun(*this, &Dashboard::UpdateInterface), 10); // call UpdateInterface if nothing is going on for 10 miliseconds

}

Dashboard::~Dashboard(void) {
    //delete currentScene;
    //delete tempInfo;
    //delete currentTime;
} 

void Dashboard::init() {
    // Initialize the hardware first
    this->BuildInterface();
    data_logger->init();
    /*
     try {
        Glib::Thread *const OBDScanner = Glib::Thread::create(sigc::mem_fun(), TRUE); // Create ECUscan's Thread
        Glib::Thread *const DataLoggerScanner = Glib::Thread::create(sigc::mem_fun(), TRUE); // Create Data Logger's Thread
    }
    catch(ThreadError e) {
        cerr << "could not create threads\r";
    }
     *
     */
}

bool Dashboard::UpdateInterface() {
    while(!drawQueue.empty()) {
        tempInfo = &drawQueue.front();
        time_t theTime;
        time(&theTime);
        currentTime = localtime(&theTime);
        tm *tempTime;
        *tempTime = tempInfo->getTime();
        double timeDif = currentTime->tm_sec - tempTime->tm_sec;
        if(timeDif < 0.5)
            queue_draw();
            drawQueue.pop(); // Pop the data out of the queue no matter what
        }
    return true;
}

// Draw the screen
bool Dashboard::on_expose_event(GdkEventExpose* event) {
    Glib::RefPtr<Gdk::Window> window = get_window();
    if(window) {
        totalDrawableWidth = get_allocation().get_width();
        totalDrawableHeight = get_allocation().get_height();
        cr = window->create_cairo_context();   
        if(initialize) {
             if(event) {
                cr->rectangle(event->area.x, event->area.y, event->area.width, event->area.height);
                cr->clip();
                //currentScene->Update(cr, tempInfo->getID(), tempInfo->getValue());
                cout << "Initialization done\n";
                cout << "Updating gauge\n";
                currentScene->Update(cr, 2, 50);
                cout << "Updated the gauge\n\n\n";
            }
        }
        else {
            // Pass the cairo context to the curent Scene
            //MessageCenter = new Diagnostic(totalDrawableWidth,  totalDrawableHeight);
            BuildInterface();
            currentScene->init(cr);
           // cout << "Updating gauge\n";
            //currentScene->Update(cr, 0, 20);
            //cout << "Done\n";
            //currentScene->Update(cr, 2, 50);
            initialize = true;           
        }
    }
    cr.clear();
    return true;
}
/** 
 * This function reads the Interface definition file and builds the
 * custom interface from it
 */
void Dashboard::BuildInterface() {
    //Parse in the the interface xml file
    list <Gauge *> Gauges;
    string NeedlePATH, SensType, DisplayType;
    string MinAng, MaxAng, MinVal, MaxVal;
    string NXpos, NYpos;
    string appendChars;
    int ID;
    
    double OutlineColor[3], textColor[3];
    string slant, weight, font, size, lineWeight;

    bool fileGood = true;
    int i, myIterator = 0;
    int gaugeType;
    XMLNode xMainNode = XMLNode::openFileHelper("Interface.xml","DVD");
    int NumScene = xMainNode.nChildNode("SCENE");
    string initialScene = xMainNode.getChildNode("INITIAL").getAttribute("scene");
    
    list <Scene_Template *>::iterator It;
    for(i=0; i < NumScene; i++) {
	XMLNode xNode = xMainNode.getChildNode("SCENE", &myIterator);
	string SceneName = xNode.getAttribute("name");
	string BrImagePATH = xNode.getChildNode("IMAGE").getAttribute("path");
	int NumGauge = xNode.nChildNode("GAUGE");
	int j, myIterator2 = 0;
	for(j=0; j<NumGauge; j++) {
            XMLNode xNode2 = xNode.getChildNode("GAUGE", &myIterator2);
            NeedlePATH = xNode2.getChildNode("IMAGE").getAttribute("path");  // only allow PNG files
            DisplayType = xNode2.getAttribute("displaytype");
            ID = atof(xNode2.getAttribute("id"));
            NXpos = xNode2.getChildNode("POSITION").getAttribute("Xpos");
            NYpos = xNode2.getChildNode("POSITION").getAttribute("Ypos");
            if(UptoLower(DisplayType) == "analog")
                gaugeType = 1; 
            if(UptoLower(DisplayType) == "textual")
                gaugeType = 0;

                switch(gaugeType) {
                    case 0:
                        appendChars = xNode2.getChildNode("FORMAT").getAttribute("appendedUnits");
                        slant = xNode2.getChildNode("FORMAT").getAttribute("fontslant");
                        weight = xNode2.getChildNode("FORMAT").getAttribute("fontweight");
                        font = xNode2.getChildNode("FORMAT").getAttribute("font");
                        size = xNode2.getChildNode("FORMAT").getAttribute("fontsize");
                        textColor[0] = atof(xNode2.getChildNode("COLOR").getAttribute("red"));
                        textColor[1] = atof(xNode2.getChildNode("COLOR").getAttribute("green"));
                        textColor[2] = atof(xNode2.getChildNode("COLOR").getAttribute("blue"));
                        if(xNode2.getChildNode("OUTLINE").getAttribute("bool") == "true") {
                            lineWeight = xNode2.getChildNode("OUTLINE").getAttribute("lineweight");
                            OutlineColor[0] = atof(xNode2.getChildNode("OUTLINE").getAttribute("red"));
                            OutlineColor[1] = atof(xNode2.getChildNode("OUTLINE").getAttribute("green"));
                            OutlineColor[2] = atof(xNode2.getChildNode("OUTLINE").getAttribute("blue"));
                            Gauges.push_back(new TextualGauge(ID,appendChars,atof(NXpos.c_str()),atof(NYpos.c_str()),slant,weight,font,atof(size.c_str()), textColor, OutlineColor, atof(lineWeight.c_str())));
                            fileGood = true;
                        }
                        else {
                            Gauges.push_back(new TextualGauge(ID,appendChars,atof(NXpos.c_str()),atof(NYpos.c_str()),slant,weight,font,atof(size.c_str()),textColor));
                            fileGood = true;
                        }                        
                        break;
                    case 1:
                        if(FileExists(NeedlePATH) && checkFileType(NeedlePATH, "png")) {
                            MinAng = xNode2.getChildNode("ROTATION").getAttribute("Minang");
                            MaxAng = xNode2.getChildNode("ROTATION").getAttribute("Maxang");
                            MinVal = xNode2.getChildNode("VALUE").getAttribute("Minval");
                            MaxVal = xNode2.getChildNode("VALUE").getAttribute("Maxval");
                            Gauges.push_back(new AnalogGauge(ID, NeedlePATH, atof(MinAng.c_str()), atof(MaxAng.c_str()), atof(NXpos.c_str()), atof(NYpos.c_str()), atof(MinVal.c_str()), atof(MaxVal.c_str())));
                            fileGood = true;
                        }
                        else
                            fileGood = false;
                        break;
                    default: fileGood = false;
                }
                if(!fileGood)
                    break;
        } // end of Gauge loop
        if(fileGood) {
            Scenes.push_back(new Scene(i,SceneName, NumGauge, Gauges, BrImagePATH));
            nScenes++;
            if(UptoLower(SceneName) == UptoLower(initialScene)) {
                It = Scenes.end();
                It--;
                currentScene = *It;
            }
        }            
        fileGood = true;
        cout << endl;
    } // end of Scene loop
    if(Scenes.empty()) {
        Scenes.push_back(new Default(0,totalDrawableWidth, totalDrawableHeight));
        It = Scenes.end();
        It--;
        currentScene = *It;
    } 
}

void Dashboard::changeScene(int Scene_ID) {
    // Use function to change the display scene 
    list <Scene_Template *>::iterator It;
    for(It=Scenes.begin(); It!=Scenes.end() && (*It)->getID()!=Scene_ID; It++)
        ;
    if(It != Scenes.end()) {
        currentScene = *It;  // will this work right.  will it copy the memory location that "IT" is pointing to into "currentScene"
        currentScene->init(cr);
    }
}

// Utility function
// Check to see if the file exist
bool Dashboard::FileExists(string FileName) {
    ifstream inp(FileName.c_str(), ifstream::in);
    inp.close();
    if(inp.fail()) {
        inp.clear(ios::failbit);
        return false;
    }
    else
        return true;
}

// Utility function
// check to see if the file has the right tye of file extension
bool Dashboard::checkFileType(string File, string FileExtension) {
    string ext = "";
    string str = File;
    int dotIndex = 0;
    string::size_type loc = File.find(FileExtension);
    
    if(loc != string::npos) {
        return true;  //file is good
    }
    else {
        return false;
    }
}

// Utility function
// Converts all upper cases characters to lower case
string Dashboard::UptoLower(string str) {
    for(int i=0; i<str.length(); i++) {
        if(isupper(str[i]))
            str[i] = tolower(str[i]);
    }
    return str;
}
